@@ -1,6 +1,14 @@
 import unittest
 
-class TestWhoConfig(unittest.TestCase):
+class _Base(unittest.TestCase):
+
+    def failUnless(self, predicate, message=''):
+        self.assertTrue(predicate, message) # Nannies go home!
+
+    def failIf(self, predicate, message=''):
+        self.assertFalse(predicate, message) # Nannies go home!
+
+class TestWhoConfig(_Base):
 
     def _getTargetClass(self):
         from repoze.who.config import WhoConfig
@@ -39,7 +47,7 @@ class TestWhoConfig(unittest.TestCase):
         self.assertEqual(len(config.mdproviders), 0)
 
     def test_parse_empty_file(self):
-        from StringIO import StringIO
+        from repoze.who._compat import StringIO
         config = self._makeOne()
         config.parse(StringIO())
         self.assertEqual(config.request_classifier, None)
@@ -222,6 +230,9 @@ class TestWhoConfig(unittest.TestCase):
         self.failUnless(isinstance(foo, DummyPlugin))
         self.assertEqual(foo.iface, 'iface')
         self.assertEqual(foo.name, 'name')
+        self.assertEqual(foo.template, '%(template)s')
+        self.assertEqual(foo.template_with_eq,
+                         'template_with_eq = %(template_with_eq)s')
 
 class DummyPlugin:
     def __init__(self, **kw):
@@ -340,9 +351,11 @@ MAKE_PLUGIN_ARG_NAMES = """\
 use = repoze.who.tests.test_config:DummyPlugin
 name = name
 iface = iface
+template = %%(template)s
+template_with_eq = template_with_eq = %%(template_with_eq)s
 """
 
-class TestConfigMiddleware(unittest.TestCase):
+class TestConfigMiddleware(_Base):
     _tempdir = None
 
     def setUp(self):
@@ -403,13 +416,14 @@ class TestConfigMiddleware(unittest.TestCase):
         logfile = os.path.join(self._tempdir, 'who.log')
         global_conf = {'here': '/'}
         middleware = factory(app, global_conf, config_file=path,
-                             log_file=logfile)
-        self.assertEqual(middleware.logger.getEffectiveLevel(), logging.INFO)
+                             log_file=logfile, log_level=logging.WARN)
+        self.assertEqual(middleware.logger.getEffectiveLevel(), logging.WARN)
         handlers = middleware.logger.handlers
         self.assertEqual(len(handlers), 1)
         self.failUnless(isinstance(handlers[0], logging.StreamHandler))
         self.assertEqual(handlers[0].stream.name, logfile)
         logging.shutdown()
+        handlers[0].stream.close()
 
     def test_sample_config_wo_log_file(self):
         import logging
@@ -419,13 +433,13 @@ class TestConfigMiddleware(unittest.TestCase):
         path = self._getTempfile(SAMPLE_CONFIG)
         global_conf = {'here': '/'}
         middleware = factory(app, global_conf, config_file=path)
-        self.assertEqual(middleware.logger.getEffectiveLevel(), 0)
+        self.assertEqual(middleware.logger.getEffectiveLevel(), logging.INFO)
         handlers = middleware.logger.handlers
         self.assertEqual(len(handlers), 1)
         self.failUnless(isinstance(handlers[0], NullHandler))
         logging.shutdown()
 
-class NullHandlerTests(unittest.TestCase):
+class NullHandlerTests(_Base):
 
     def _getTargetClass(self):
         from repoze.who.config import NullHandler
@@ -443,7 +457,7 @@ class NullHandlerTests(unittest.TestCase):
         handler = self._makeOne()
         handler.emit(object())
 
-class Test_make_api_factory_with_config(unittest.TestCase):
+class Test_make_api_factory_with_config(_Base):
     _tempdir = None
     _warning_filters = None
 
@@ -458,16 +472,6 @@ class Test_make_api_factory_with_config(unittest.TestCase):
             import warnings
             warnings.filters[:] = self._warning_filters
 
-    def _filterWarnings(self):
-        global __warningregistry__
-        try:
-            del __warningregistry__
-        except NameError:
-            pass
-        import warnings
-        self._warning_filters = warnings.filters[:]
-        warnings.filterwarnings("ignore")
-
     def _getFactory(self):
         from repoze.who.config import make_api_factory_with_config
         return make_api_factory_with_config
@@ -484,34 +488,34 @@ class Test_make_api_factory_with_config(unittest.TestCase):
         return path
 
     def test_bad_config_filename(self):
-        global __warningregistry__
-        self._filterWarnings()
-        factory = self._getFactory()
-        path = '/nonesuch/file/should/exist'
-        global_conf = {'here': '/'}
-        api_factory = factory(global_conf, config_file=path)
-        self.assertEqual(len(api_factory.identifiers), 0)
-        self.assertEqual(len(api_factory.authenticators), 0)
-        self.assertEqual(len(api_factory.challengers), 0)
-        self.assertEqual(len(api_factory.mdproviders), 0)
-        self.assertEqual(api_factory.remote_user_key, 'REMOTE_USER')
-        self.failUnless(api_factory.logger is None)
-        self.failUnless(__warningregistry__)
+        import warnings
+        with warnings.catch_warnings(record=True) as warned:
+            factory = self._getFactory()
+            path = '/nonesuch/file/should/exist'
+            global_conf = {'here': '/'}
+            api_factory = factory(global_conf, config_file=path)
+            self.assertEqual(len(api_factory.identifiers), 0)
+            self.assertEqual(len(api_factory.authenticators), 0)
+            self.assertEqual(len(api_factory.challengers), 0)
+            self.assertEqual(len(api_factory.mdproviders), 0)
+            self.assertEqual(api_factory.remote_user_key, 'REMOTE_USER')
+            self.failUnless(api_factory.logger is None)
+            self.failUnless(warned)
 
     def test_bad_config_content(self):
-        global __warningregistry__
-        self._filterWarnings()
-        factory = self._getFactory()
-        path = self._getTempfile('this is not an INI file')
-        global_conf = {'here': '/'}
-        api_factory = factory(global_conf, config_file=path)
-        self.assertEqual(len(api_factory.identifiers), 0)
-        self.assertEqual(len(api_factory.authenticators), 0)
-        self.assertEqual(len(api_factory.challengers), 0)
-        self.assertEqual(len(api_factory.mdproviders), 0)
-        self.assertEqual(api_factory.remote_user_key, 'REMOTE_USER')
-        self.failUnless(api_factory.logger is None)
-        self.failUnless(__warningregistry__)
+        import warnings
+        with warnings.catch_warnings(record=True) as warned:
+            factory = self._getFactory()
+            path = self._getTempfile('this is not an INI file')
+            global_conf = {'here': '/'}
+            api_factory = factory(global_conf, config_file=path)
+            self.assertEqual(len(api_factory.identifiers), 0)
+            self.assertEqual(len(api_factory.authenticators), 0)
+            self.assertEqual(len(api_factory.challengers), 0)
+            self.assertEqual(len(api_factory.mdproviders), 0)
+            self.assertEqual(api_factory.remote_user_key, 'REMOTE_USER')
+            self.failUnless(api_factory.logger is None)
+            self.failUnless(warned)
 
     def test_sample_config_no_logger(self):
         factory = self._getFactory()
